const pool = global.db_pool;
if (!pool) {
	throw new Error('Database pool is not defined');
}

const express = require('express');
const app = new express.Router();

const analytics = require('../analytics');

// Admin routes (Authenticated)

app.get('/v1/admin/routes', (req, res) => { // Get all routes
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM routes').then((rows) => {
			res.json(rows);
		}).catch(err => {
			console.error('Error getting routes:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.get('/v1/admin/route/:id', (req, res) => { // Get route
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM routes WHERE id = ?', [req.params.id]).then((rows) => {
			const row = rows[0];
			if (!row) {
				res.status(404).json({ error: 'Not Found' });
				return;
			}
			res.json(row);
		}).catch(err => {
			console.error('Error getting route:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.post('/v1/admin/route', (req, res) => { // Create a new route
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	const server = req.body.server;
	const port = req.body.port;
	const auth = req.body.auth || "from-astrocom";
	const secret = req.body.secret || crypto.randomBytes(15).toString('hex');
	const block_start = req.body.block_start;
	const block_length = req.body.block_length || 9999;
	const apiKey = crypto.randomBytes(32).toString('hex');
	const contact = req.body.contact || "Unknown";
	// Validate all inputs exist
	if (!server || !port || !block_start) {
		res.status(400).json({ error: 'Bad Request' });
		return;
	}
	// Check if route already exists (OR conditions on server, and block range)
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM routes WHERE block_start <= ? AND block_start + block_length >= ?', [block_start, block_start]).then((rows) => {
			const row = rows[0];
			if (row) {
				res.status(409).json({ error: 'Conflict' });
				return;
			} else {
				conn.query('INSERT INTO routes (server, port, auth, secret, block_start, block_length, apiKey, contact) VALUES (?, ?, ?, ?, ?, ?, ?, ?)',
					[server, port, auth, secret, block_start, block_length, apiKey, contact]).then(() => {
						res.status(201).json({ message: 'Created' });
					}).catch(err => {
						console.error('Error creating route:', err);
						res.status(500).json({ error: 'Internal server error' });
					});
			}
		}).catch(err => {
			console.error('Error checking for existing route:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.put('/v1/admin/route/:id', (req, res) => { // Update a route
	// Check if authenticated
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	// Check if route exists
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM routes WHERE id = ?', [req.params.id]).then((rows) => {
			const row = rows[0];
			if (!row) {
				res.status(404).json({ error: 'Not Found' });
				return;
			}
			// Update route
			const server = req.body.server || row.server;
			const port = req.body.port || row.port;
			const auth = req.body.auth || row.auth;
			const secret = req.body.secret || row.secret;
			const block_start = req.body.block_start || row.block_start;
			const block_length = req.body.block_length || row.block_length;
			const contact = req.body.contact || row.contact;
			console.log(`Updating ${req.params.id} to ${server}:${port} with ${auth}:${secret} for ${block_start} - ${block_start + block_length}. Contact: ${contact}`);
			conn.query('UPDATE routes SET server = ?, port = ?, auth = ?, secret = ?, block_start = ?, block_length = ?, contact = ? WHERE id = ?',
				[server, port, auth, secret, block_start, block_length, contact, req.params.id]).then(() => {
					res.json({ message: 'Updated' });
				}).catch(err => {
					console.error('Error updating route:', err);
					res.status(500).json({ error: 'Internal server error' });
				});
		}).catch(err => {
			console.error('Error getting route:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.delete('/v1/admin/route/:id', (req, res) => { // Delete a route
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	pool.getConnection().then(async conn => {
		await conn.query('DELETE FROM directory WHERE route = ?', [req.params.id])
		conn.query('DELETE FROM routes WHERE id = ?', [req.params.id]).then(() => {
			res.json({ message: 'Deleted' });
		}).catch(err => {
			console.error('Error deleting route:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.delete('/v1/admin/directory/:number', (req, res) => { // Delete a directory entry
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	const number = Number(req.params.number);
	if (!number) {
		res.status(400).json({ error: 'Bad Request' });
		return;
	}
	pool.getConnection().then(conn => {
		conn.query('DELETE FROM directory WHERE number = ?', [number]).then(() => {
			res.status(200).json({ message: 'Deleted' });
		}).catch(err => {
			console.error('Error deleting directory entry:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.get("/v1/admin/callLogs", (req, res) => {
	if (!req.session.adminAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	// if ?page is set, return 100 results from that page, if no page assume page 1
	const page = Number(req.query.page) || 1;
	const offset = (page - 1) * 100;
	// Get full count of call logs to calculate total pages
	pool.getConnection().then(conn => {
		conn.query("SELECT COUNT(*) as count FROM callLogs").then((rows) => {
			const totalPages = Math.ceil(rows[0].count / 100);
			conn.query("SELECT * FROM callLogs ORDER BY timestamp DESC LIMIT 100 OFFSET ?", [offset]).then((rows) => {
				res.json({ totalPages, page, data: rows });
			}).catch(err => {
				console.error('Error getting call logs:', err);
				res.status(500).json({ error: 'Internal server error' });
			});
		}).catch(err => {
			console.error('Error getting call log count:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

// User routes (Authenticated)

app.get('/v1/user/route', (req, res) => { // Get route
	if (!req.session.userAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	res.json(req.session.userData);
});

app.put('/v1/user/route', (req, res) => { // Update route
	if (!req.session.userAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	if (!req.session.userData.apiKey) {
		req.session.destroy(); // Something weird happened, destroy session
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	// Does not allow for ID to be specified, always update current users route
	const server = req.body.server || req.session.userData.server;
	const port = req.body.port || req.session.userData.port;
	const auth = req.body.auth || req.session.userData.auth;
	const secret = req.body.secret || req.session.userData.secret;
	pool.getConnection().then(conn => {
		conn.query('UPDATE routes SET server = ?, port = ?, auth = ?, secret = ? WHERE apiKey = ?',
			[server, port, auth, secret, req.session.userData.apiKey]).then(() => {
				req.session.userData.server = server;
				req.session.userData.port = port;
				req.session.userData.auth = auth;
				req.session.userData.secret = secret;
				res.json({ message: 'Updated' });
			}).catch(err => {
				console.error('Error updating route:', err);
				res.status(500).json({ error: 'Internal server error' });
			}).finally(() => {
				conn.release();
			});
	});
});

app.get('/v1/user/directory', (req, res) => { // Get directory entries created by user
	if (!req.session.userAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM directory WHERE route = ?', [req.session.userData.id]).then((rows) => {
			res.json(rows);
		}).catch(err => {
			console.error('Error getting routes:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.post('/v1/user/directory', (req, res) => { // Create a new directory entry
	// Check if authenticated
	if (!req.session.userAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	// Check that the number is within the block range for the current user
	var number = Number(req.body.number);
	var name = String(req.body.name);
	if (!number || !name) {
		res.status(400).json({ error: 'Bad Request' });
		return;
	}

	if (number < req.session.userData.block_start || number > req.session.userData.block_start + req.session.userData.block_length) {
		res.status(403).json({ error: 'Forbidden' });
		return;
	}

	// Remove html
	name = require("escape-html")(name);

	const route = req.session.userData.id;
	// If number already exists, update, otherwise insert
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM directory WHERE number = ? AND route = ?', [number, route]).then((rows) => {
			const row = rows[0];
			if (row) {
				conn.query('UPDATE directory SET name = ? WHERE number = ? AND route = ?',
					[name, number, route]).then(() => {
						res.json({ message: 'Updated' });
					}).catch(err => {
						console.error('Error updating directory entry:', err);
						res.status(500).json({ error: 'Internal server error' });
					});
			} else {
				conn.query('INSERT INTO directory (number, name, route) VALUES (?, ?, ?)',
					[number, name, route]).then(() => {
						res.status(201).json({ message: 'Created' });
					}).catch(err => {
						console.error('Error creating directory entry:', err);
						res.status(500).json({ error: 'Internal server error' });
					});
			}
		}).catch(err => {
			console.error('Error checking for existing directory entry:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.delete('/v1/user/directory/:number', (req, res) => { // Delete a directory entry
	if (!req.session.userAuthenticated) {
		res.status(401).json({ error: 'Unauthorized' });
		return;
	}
	const number = Number(req.params.number);
	if (!number) {
		res.status(400).json({ error: 'Bad Request' });
		return;
	}
	pool.getConnection().then(conn => {
		conn.query('DELETE FROM directory WHERE number = ? AND route = ?', [number, req.session.userData.id]).then(() => {
			res.status(200).json({ message: 'Deleted' });
		}).catch(err => {
			console.error('Error deleting directory entry:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

// Public routes

app.get("/v1/directory", (req, res) => {
	pool.getConnection().then(conn => {
		conn.query("SELECT * FROM directory").then((rows) => {
			res.json(rows);
		}).catch(err => {
			console.error('Error getting directory:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.get("/analytics", (req, res) => {
	pool.getConnection().then(conn => {
		conn.query("SELECT * FROM analytics").then((total) => {
			conn.query("SELECT * FROM dailyAnalytics").then((daily) => {
				// Find the latest date and add "current:true" to it
				var latest = { tag_date: "1970-01-01", count: 0 };
				daily.forEach((entry) => {
					if (entry.tag_date > latest.tag_date) {
						latest = entry;
					}
				});
				latest.current = true;
				res.json({ total, daily });
			}).catch(err => {
				console.error('Error getting daily analytics:', err);
				res.status(500).send('Internal server error');
			});
		}).catch(err => {
			console.error('Error getting analytics:', err);
			res.status(500).send('Internal server error');
		}).finally(() => {
			conn.release();
		});
	});
});

app.get("/v1/checkAvailability/:number", (req, res) => {
	// Check if the number is 7 digits
	const number = Number(req.params.number);
	if (number < 2000000 || number > 9999999) {
		res.status(400).json({ error: `Number is outside valid range` });
		return;
	}
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM routes WHERE block_start <= ? AND block_start + block_length >= ?', [number, number]).then((rows) => {
			const row = rows[0];
			if (row) {
				res.json({ available: false, block: row.block_start });
			} else {
				res.json({ available: true });
			}
		}).catch(err => {
			console.error('Error getting route:', err);
			res.status(500).json({ error: 'Internal server error' });
		}).finally(() => {
			conn.release();
		});
	});
});

app.get("/healthcheck", (req, res) => {
	// Check ability to connect to database with select * from routes
	pool.getConnection().then(conn => {
		conn.query('SELECT * FROM routes').then(() => {
			res.status(200).send('OK');
		}).catch(err => {
			console.error('Error checking health:', err);
			res.status(500).send('Internal server error');
		}).finally(() => {
			conn.release();
		});
	});
});

// AstroCom Asterisk routes (Authenticated)

app.get('/v1/route/:apiKey/:ani/:number', (req, res) => {
	const apiKey = req.params.apiKey;
	const number = Number(req.params.number);
	const ani = Number(req.params.ani);
	pool.getConnection().then(conn => {
		//conn.query("SELECT * FROM routes WHERE apiKey = ? AND block_start <= ? AND block_start + block_length >= ?", [apiKey, ani, ani]).then((rows) => {
		conn.query("SELECT * FROM routes WHERE apiKey = ?", [apiKey]).then((rows) => { // We'll try this Nick, if it doesn't work we'll go back to the original
			const row = rows[0];
			// If no row or error, return 401
			if (!row) {
				res.status(401).send(`${process.env.MSG_ROUTE_ADDRESS}/401`)
				return;
			}
			conn.query('SELECT * FROM routes WHERE block_start <= ? AND block_start + block_length >= ?', [number, number]).then((rows) => {
				const row = rows[0];
				if (row) {
					// Check if the ANI is within the block range
					// If it is, return `local`
					console.log(`New Call: ${ani} -> ${number}`);
					analytics.logCall(ani, number);
					// incriment estCallsMade analytics
					analytics.addAnalytic("estCallsMade");
					analytics.dailyAnalytic("dailyCallsMade");
					if (ani >= row.block_start && ani <= row.block_start + row.block_length) {
						res.status(200).send('local');
					} else {
						res.status(200).send(`IAX2/${row.auth}:${row.secret}@${row.server}:${row.port}/${number}`);
					}
				} else {
					res.status(404).send(`${process.env.MSG_ROUTE_ADDRESS}/404`);
				}
			}).catch(err => {
				console.error('Error getting route:', err);
				res.status(500).send(`${process.env.MSG_ROUTE_ADDRESS}/500`)
			});
		}).catch(err => {
			console.error(err);
			res.status(401).send(`${process.env.MSG_ROUTE_ADDRESS}/401`)
		}).finally(() => {
			conn.release();
		});
	});
});

app.get('/v1', (req, res) => { // Backwards compatibility with TandmX cause why not, it's easy
	const apiKey = req.query.auth;
	const number = Number(req.query.number);
	const ani = Number(req.query.ani);
	pool.getConnection().then(conn => {
		conn.query("SELECT * FROM routes WHERE apiKey = ? AND block_start <= ? AND block_start + block_length >= ?", [apiKey, ani, ani]).then((rows) => {
			const row = rows[0];
			// If no row or error, return 401
			if (!row) {
				res.status(401).send(`${process.env.MSG_ROUTE_ADDRESS}/401`)
				return;
			}
			conn.query('SELECT * FROM routes WHERE block_start <= ? AND block_start + block_length >= ?', [number, number]).then((rows) => {
				const row = rows[0];
				if (row) {
					// Check if the ANI is within the block range
					// If it is, return `local`
					console.log(`New Call: ${ani} -> ${number}`);
					analytics.logCall(ani, number);
					analytics.addAnalytic("estCallsMade");
					analytics.dailyAnalytic("dailyCallsMade");
					if (ani >= row.block_start && ani <= row.block_start + row.block_length) {
						res.status(200).send('local');
					} else {
						res.status(200).send(`IAX2/${row.auth}:${row.secret}@${row.server}:${row.port}/${number}`);
					}
				} else {
					res.status(404).send(`${process.env.MSG_ROUTE_ADDRESS}/404`);
				}
			}).catch(err => {
				console.error('Error getting route:', err);
				res.status(500).send(`${process.env.MSG_ROUTE_ADDRESS}/500`)
			});
		}).catch(err => {
			console.error(err);
			res.status(401).send(`${process.env.MSG_ROUTE_ADDRESS}/401`)
		}).finally(() => {
			conn.release();
		});
	});
});

// Management Routes (Like restarting the server)

app.post("/api/v1/manage/restart", (req, res) => {
	// Check Authorization header against process.env.MANAGEMENT_AUTH
	if (req.headers.authorization !== (process.env.MANAGEMENT_AUTH || crypto.randomBytes(32).toString('hex'))) { // Default to random in case we forget to configure it
		res.status(401).send("Unauthorized");
		return;
	}
	res.status(200).send("Restarting server...");
	setTimeout(() => {
		process.exit(0);
	}, 500);
});

module.exports = app;